/*************************************************************************
 * $Id: MyCallback.h,v 1.10 2006/03/16 17:49:17 vtolkatc Exp $
 *
 * This file is a part of the "Asynchronous Client" sample which implements 
 * all the code to demonstrate the use of the Symphony API within a client 
 * application in an asynchronous manner.
 *
 * Copyright (c) 2001-2006 Platform Computing Corporation. 
 * Accelerating Intelligence(TM). All rights reserved.
 *
 * This exposed source code is the confidential and proprietary property
 * of Platform Computing Corporation. Your right to use is strictly limited
 * by the terms of the license agreement entered into with Platform Computing
 * Corporation.
 *
 *************************************************************************/

#include "soam.h"

using namespace soam;
using namespace std;

#ifndef WIN32
#include <pthread.h>
#else
#include <windows.h>
#endif

class MySessionCallback : 
    public SessionCallback
{
    public:
        MySessionCallback()
            :m_tasksReceived(0), m_exception(false) 
        {

#ifndef WIN32
            pthread_mutexattr_t attr;
            pthread_mutexattr_init( &attr );
            pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE ); 

            pthread_mutex_init( &m_mutex, &attr );

            pthread_mutexattr_destroy( &attr );
#else
            InitializeCriticalSection(&m_criticalSection); 
#endif

            //cout << "Callback created ... " << endl;
        }
        virtual ~MySessionCallback() 
        {
#ifndef WIN32
            pthread_mutex_destroy( &m_mutex );
#else
            DeleteCriticalSection(&m_criticalSection);
#endif
        }

        /////////////////////////////////////////////////////////
        // This handler is called once any exception occurs
        // within the scope of the session.
        // ======================================================
        virtual void onException(SoamException &exception) throw()
        {
            cout << "An exception occured on the callback.\nDetails : " <<  exception.what() << endl;

#ifndef WIN32
                pthread_mutex_lock( &m_mutex);
#else
                EnterCriticalSection(&m_criticalSection); 
#endif
                m_exception = true;
#ifndef WIN32
                pthread_mutex_unlock( &m_mutex);
#else
                LeaveCriticalSection(&m_criticalSection);
#endif

        }

        /////////////////////////////////////////////////////////
        // This handler is called once a message is returned 
        // from the system when a task completes.
        // ======================================================
        void onResponse(TaskOutputHandlePtr &output) throw()
        {
            try
            {
                // check for success of task
                if (true == output->isSuccessful())
                {
                    // get the message returned from the service
                    MyMessage outMsg;
                    output->getMessage(&outMsg);

                    // display content of reply
                    // cout << "Task Succeeded [" <<  output->getId() << "]" << endl;
                    // cout << "Integer Value : " << outMsg.getInt() << endl;
                    cout << outMsg.getString() << endl;
                }
                else
                {
                    // get the exception associated with this task
                    SoamExceptionPtr ex = output->getException();
                    cout << "Task Failed : " << ex->what() << endl;
                }

            }
            catch(SoamException &exception)
            {
                cout << "Exception occured in OnResponse() : " << exception.what() << endl;
            }

            // Update counter used to synchronize the controlling thread 
            // with this callback object
#ifndef WIN32
                pthread_mutex_lock( &m_mutex);
#else
                EnterCriticalSection(&m_criticalSection); 
#endif
                ++m_tasksReceived;
#ifndef WIN32
                pthread_mutex_unlock( &m_mutex);
#else
                LeaveCriticalSection(&m_criticalSection);
#endif
        }
        
        inline long getReceived() 
        { 
            return m_tasksReceived; 
        }

        inline bool getDone()
        {
            return m_exception;
        }
                   
    private:
#ifndef WIN32
        pthread_mutex_t m_mutex;
#else
        CRITICAL_SECTION m_criticalSection;
#endif

        long m_tasksReceived;
        bool m_exception;
};
