
/* This is the implementation of the IVOP service.
 */
 
#include "stdafx.h"
#include <stdio.h>
#include "soam.h"
#include "MyMessage.h"

using namespace soam;
using namespace std;

extern "C" {
// from libmatappl.a 

extern int heston1dim_(double *,
		       double *,
		       double *,
		       double *,
		       double *,
		       double *,
		       double *,
		       double *,
		       double *,
		       double *);

static int read_protocol_message(const char* ,
				 double *,
				 double *,
				 double *,
				 double *,
				 double *,
				 double *,
				 double *,
				 double *,
				 double *);

}
class MyServiceContainer : public ServiceContainer
{
public:
    
    virtual void onInvoke (TaskContextPtr& taskContext) throw(SoamException) {
        // get the input that was sent from the client
        MyMessage inMsg;
        taskContext->getInputMessage(inMsg);
	
        // We simply echo the data back to the client
        MyMessage outMsg;
        outMsg.setInt(inMsg.getInt());

	double   dstep;  // math parameter
	double   eps;    // math parameter
	double   chi;    // math parameter 
	double   theta;  // math parameter
	double   rho;    // math parameter
	double   v0;     // fin parameter
	double   eprice; // fin parameter
	double   tau;    // fin parameter
	double   rtax;   // fin parameter
	double   option; // the result

	dstep = eps = chi = theta 
	    = rho = v0 = eprice = tau 
	    = rtax = option = 0.0;

	const char   *p = inMsg.getString();
	
	// read the input from the client specifying
	// the mathematical and financial paramenters
	// fot the experiment
	read_protocol_message(p,
			      &dstep,
			      &eps,
			      &chi,
			      &theta,
			      &rho,
			      &v0,
			      &eprice,
			      &tau,
			      &rtax);
	// solve the experiment
	heston1dim_(&dstep,
		    &eps,
		    &chi,
		    &theta,
		    &rho,
		    &v0,
		    &eprice,
		    &tau,
		    &rtax,
		    &option); // output the option value
	
	char   buf[256];
	sprintf(buf, "The option discounted value %4.4f", option);

	outMsg.setString(buf);

        // set our output message
        taskContext->setOutputMessage(outMsg);

    } /* virtual void onInvoke() */
};

int 
main(int argc, char* argv[])
{
    // return value of our service program
    int retVal = 0;

    try	{
        // Create the container and run it
        MyServiceContainer myContainer;
        myContainer.run(argc, argv);
    }
    catch(SoamException& exp) {
        // report exception to stdout
        cout << "exception caught ... " << exp.what() << endl;
	retVal = -1;
    }

    // NOTE: Although our service program will return an overall 
    // failure or success code it will always be ignored in the 
    // current revision of the middleware. 
    // The value being returned here is for consistency.
    return retVal;
} 

// parse the protocol message sent from the client
static int
read_protocol_message(const char *p,
		      double *dstep,
		      double *eps,
		      double *chi,
		      double *theta,
		      double *rho,
		      double *v0,
		      double *eprice,
		      double *tau,
		      double *rtax)
{
    float   a;  //dstep
    float   b;  //eps
    float   c;  //chi
    float   d;  //theta
    float   e;  //rho
    float   x;  //v0
    float   y;  //eprice
    float   z;  //tau
    float   u;  //rtax

    *dstep = *eps = *chi 
	= *theta = *rho 
	= *v0 = *eprice = *tau = *rtax = 0.0;

    /* scan  v0 eprice tau rtax
     */
    int cc = sscanf(p, "\
%f%f%f%f%f%f%f%f%f", &a, &b, &c, &d, &e, 
		    &x, &y, &z, &u);

    *dstep  = (double)a;
    *eps    = (double)b;
    *chi    = (double)c;
    *theta  = (double)d;
    *rho    = (double)e;
    *v0     = (double)x;
    *eprice = (double)y;
    *tau    = (double)z;
    *rtax   = (double)u;

    // debug
    FILE   *fp;
    fp = fopen("/tmp/service.log", "a+");
    fprintf(fp, "got: %s\n", p);
    fprintf(fp, "\
processed: cc = %d dstep = %4.4f eps = %4.4f chi = %4.4f theta = %4.4f rho = %4.4f v0 = %4.4f eprice = %4.4f tau = %4.4f rtax = %4.4f\n",
	    cc, *dstep, *eps, *chi, *theta, 
	    *rho, *v0, *eprice, *tau, *rtax);

    fclose(fp);

    return(0);

} /* read_protocol_message() */
