
/* Dealing with several aspects of the user inut. 
 * 1) from the terminal using the readline library 
 * 2) from a file using the stdio
 */
#include"tools.h"

extern HIST_ENTRY **history_list ();

static void
print_history(void);

/* Get the input from the terminal, if the input parameter 
 * is NULL we use the default prompt ciao-> otherwise 
 * we use the prompt specified by the caller, the caller 
 * may be asking specific questions to the user...
 */
char *
getinput(const char *prompt)
{
    static char   *p;

    zfree(p);
    if (prompt == NULL)
	prompt = "ciao->";
    
    p = readline(prompt);
    
    /* Test for EOF. 
     */
    if (p == NULL
	|| strcmp(p, "quit") == 0)
	return(NULL);

    /* If there is anything on the line, print it and remember it. 
     */
    add_history(p);
	        
    if (strcmp (p, "list") == 0)  
	print_history();

    /* return the input buffer to the caller.
     */
    return(p);

} /* main() */

/* convert the string pointed by p to a double 
 * and detect errors
 */
double 
c2d(const char *p, int *error)
{
    char     *end;
    double   z;
    
    z = strtod(p, &end);
    if (z == 0 && end == p) {
	*error = 1;
	return(-1.0);
    }

    *error = 0;
    return(z);

} /* c2d() */

/* static workers
 */

static void
print_history(void)
{
    HIST_ENTRY   **list;
    int          i;
    
    list = history_list();
    if (list) {
	for (i = 0; list[i]; i++)
	    fprintf (stderr, "%d: %s\r\n", i, list[i]->line);
    }

} /* print_history() */

/* Get the next meaningful line from the configuration
 * file, meaningful is a line that is not isspace()
 * and that does not start with a comment # character.
 */
char *
nextline_(FILE *fp)
{
    static char  buf[BUFSIZ];
    char         *p;

    if (fp == NULL)
	return(NULL);

    p = NULL;
    while (fgets(buf, BUFSIZ, fp)) {
        p = buf;
        while (isspace(*p))
            ++p;
        if (*p == '#'
            || *p == 0) {
            /* If this is the last or only 
             * line do not return the 
             * previous buffer to the caller.
             */
            p = NULL;
            continue;
        }
        break;
    }
    /* chop da \n
     */
    chop(p);
    return(p);

} /* nextline_() */

/* Perl?
 */
char *
chop(char *z)
{
    int   L;
    
    L = strlen(z);
    z[L - 1] = 0;

    return(z);

} /* chop() */

